@@ -43,15 +43,16 @@ typedef void WINDOW;
 #include "XSUB.h"
 #else
 typedef void SV;
+typedef void AV;
+typedef void HV;
 #endif
 
-static const char owl_h_fileIdent[] = "$Id$";
-
-#define BARNOWL_STRINGIFY(x) _STRINGIFY(x)
-#define _STRINGIFY(x) #x
-
-#ifndef OWL_VERSION_STRING
-#define OWL_VERSION_STRING "1.1.1"
+#ifdef  GIT_VERSION
+#define stringify(x)       __stringify(x)
+#define __stringify(x)     #x
+#define OWL_VERSION_STRING stringify(GIT_VERSION)
+#else
+#define OWL_VERSION_STRING PACKAGE_VERSION
 #endif
 
 /* Feature that is being tested to redirect stderr through a pipe. 
@@ -111,9 +112,9 @@ static const char owl_h_fileIdent[] = "$Id$";
 #define OWL_MESSAGE_DIRECTION_IN    1
 #define OWL_MESSAGE_DIRECTION_OUT   2
 
-#define OWL_MUX_READ   1
-#define OWL_MUX_WRITE  2
-#define OWL_MUX_EXCEPT 4
+#define OWL_IO_READ   1
+#define OWL_IO_WRITE  2
+#define OWL_IO_EXCEPT 4
 
 #define OWL_DIRECTION_NONE      0
 #define OWL_DIRECTION_DOWNWARDS 1
@@ -180,8 +181,6 @@ static const char owl_h_fileIdent[] = "$Id$";
 #define OWL_DEFAULT_ZAWAYMSG    "I'm sorry, but I am currently away from the terminal and am\nnot able to receive your message.\n"
 #define OWL_DEFAULT_AAWAYMSG    "I'm sorry, but I am currently away from the terminal and am\nnot able to receive your message.\n"
 
-#define OWL_INCLUDE_REG_TESTS   1  /* whether to build in regression tests */
-
 #define OWL_CMD_ALIAS_SUMMARY_PREFIX "command alias to: "
 
 #define OWL_WEBZEPHYR_PRINCIPAL "daemon.webzephyr"
@@ -200,6 +199,25 @@ static const char owl_h_fileIdent[] = "$Id$";
 
 #define LINE 2048
 
+#ifdef HAVE_LIBZEPHYR
+/* libzephyr doesn't use const, so we appease the type system with this kludge.
+ * This just casts const char * to char * in a way that doesn't yield a warning
+ * from gcc -Wcast-qual. */
+static inline char *zstr(const char *str)
+{
+  union { char *rw; const char *ro; } u;
+  u.ro = str;
+  return u.rw;
+}
+#endif
+
+/* Convert char *const * into const char *const *.  This conversion is safe,
+ * and implicit in C++ (conv.qual 4) but for some reason not in C. */
+static inline const char *const *strs(char *const *pstr)
+{
+  return (const char *const *)pstr;
+}
+
 typedef struct _owl_variable {
   char *name;
   int   type;  /* OWL_VARIABLE_* */
@@ -209,26 +227,26 @@ typedef struct _owl_variable {
   char *summary;		/* summary of usage */
   char *description;		/* detailed description */
   void *val;                    /* current value */
-  int  (*validate_fn)(struct _owl_variable *v, void *newval);
+  int  (*validate_fn)(const struct _owl_variable *v, const void *newval);
                                 /* returns 1 if newval is valid */
-  int  (*set_fn)(struct _owl_variable *v, void *newval); 
+  int  (*set_fn)(struct _owl_variable *v, const void *newval); 
                                 /* sets the variable to a value
 				 * of the appropriate type.
 				 * unless documented, this 
 				 * should make a copy. 
 				 * returns 0 on success. */
-  int  (*set_fromstring_fn)(struct _owl_variable *v, char *newval);
+  int  (*set_fromstring_fn)(struct _owl_variable *v, const char *newval);
                                 /* sets the variable to a value
 				 * of the appropriate type.
 				 * unless documented, this 
 				 * should make a copy. 
 				 * returns 0 on success. */
-  void *(*get_fn)(struct _owl_variable *v);
+  const void *(*get_fn)(const struct _owl_variable *v);
 				/* returns a reference to the current value.
 				 * WARNING:  this approach is hard to make
 				 * thread-safe... */
-  int  (*get_tostring_fn)(struct _owl_variable *v, 
-			  char *buf, int bufsize, void *val); 
+  int  (*get_tostring_fn)(const struct _owl_variable *v, 
+			  char *buf, int bufsize, const void *val); 
                                 /* converts val to a string 
 				 * and puts into buf */
   void  (*free_fn)(struct _owl_variable *v);
@@ -290,14 +308,14 @@ typedef struct _owl_cmd {	/* command */
   char *cmd_aliased_to;		/* what this command is aliased to... */
   
   /* These don't take any context */
-  char *(*cmd_args_fn)(int argc, char **argv, char *buff);  
+  char *(*cmd_args_fn)(int argc, const char *const *argv, const char *buff);  
 				/* takes argv and the full command as buff.
 				 * caller must free return value if !NULL */
   void (*cmd_v_fn)(void);	/* takes no args */
   void (*cmd_i_fn)(int i);	/* takes an int as an arg */
 
   /* The following also take the active context if it's valid */
-  char *(*cmd_ctxargs_fn)(void *ctx, int argc, char **argv, char *buff);  
+  char *(*cmd_ctxargs_fn)(void *ctx, int argc, const char *const *argv, const char *buff);  
 				/* takes argv and the full command as buff.
 				 * caller must free return value if !NULL */
   void (*cmd_ctxv_fn)(void *ctx);	        /* takes no args */
@@ -307,6 +325,7 @@ typedef struct _owl_cmd {	/* command */
 
 
 typedef struct _owl_zwrite {
+  char *zwriteline;
   char *class;
   char *inst;
   char *realm;
@@ -319,7 +338,7 @@ typedef struct _owl_zwrite {
 } owl_zwrite;
 
 typedef struct _owl_pair {
-  char *key;
+  const char *key;
   char *value;
 } owl_pair;
 
@@ -333,7 +352,7 @@ typedef struct _owl_message {
 #endif
   struct _owl_fmtext_cache * fmtext;
   int delete;
-  char *hostname;
+  const char *hostname;
   owl_list attributes;            /* this is a list of pairs */
   char *timestr;
   time_t time;
@@ -389,9 +408,9 @@ typedef struct _owl_regex {
 } owl_regex;
 
 typedef struct _owl_filterelement {
-  int (*match_message)(struct _owl_filterelement *fe, owl_message *m);
+  int (*match_message)(const struct _owl_filterelement *fe, const owl_message *m);
   /* Append a string representation of the filterelement onto buf*/
-  void (*print_elt)(struct _owl_filterelement *fe, GString *buf);
+  void (*print_elt)(const struct _owl_filterelement *fe, GString *buf);
   /* Operands for and,or,not*/
   struct _owl_filterelement *left, *right;
   /* For regex filters*/
@@ -405,14 +424,14 @@ typedef struct _owl_filter {
   owl_filterelement * root;
   int fgcolor;
   int bgcolor;
-  int cachedmsgid;  /* cached msgid: should move into view eventually */
 } owl_filter;
 
 typedef struct _owl_view {
   char *name;
   owl_filter *filter;
   owl_messagelist ml;
-  owl_style *style;
+  const owl_style *style;
+  int cachedmsgid;
 } owl_view;
 
 typedef struct _owl_history {
@@ -423,27 +442,12 @@ typedef struct _owl_history {
   int repeats;
 } owl_history;
 
-typedef struct _owl_editwin {
-  char *buff;
-  owl_history *hist;
-  int bufflen;
-  int allocated;
-  int buffx, buffy;
-  int topline;
-  int winlines, wincols, fillcol, wrapcol;
-  WINDOW *curswin;
-  int style;
-  int lock;
-  int dotsend;
-  int echochar;
-
-  char *command;
-  void (*callback)(struct _owl_editwin*);
-  void *cbdata;
-} owl_editwin;
+typedef struct _owl_editwin owl_editwin;
+typedef struct _owl_editwin_excursion owl_editwin_excursion;
 
 typedef struct _owl_keybinding {
-  int  *j;			/* keypress stack (0-terminated) */  
+  int  *keys;			/* keypress stack */
+  int   len;                    /* length of stack */
   int   type;			/* command or function? */
   char *desc;			/* description (or "*user*") */
   char *command;		/* command, if of type command */
@@ -454,7 +458,7 @@ typedef struct _owl_keymap {
   char     *name;		/* name of keymap */
   char     *desc;		/* description */
   owl_list  bindings;		/* key bindings */
-  struct _owl_keymap *submap;	/* submap */
+  const struct _owl_keymap *submap;	/* submap */
   void (*default_fn)(owl_input j);	/* default action (takes a keypress) */
   void (*prealways_fn)(owl_input  j);	/* always called before a keypress is received */
   void (*postalways_fn)(owl_input  j);	/* always called after keypress is processed */
@@ -462,7 +466,7 @@ typedef struct _owl_keymap {
 
 typedef struct _owl_keyhandler {
   owl_dict  keymaps;		/* dictionary of keymaps */
-  owl_keymap *active;		/* currently active keymap */
+  const owl_keymap *active;		/* currently active keymap */
   int	    in_esc;		/* escape pressed? */
   int       kpstack[OWL_KEYMAP_MAXSTACK+1]; /* current stack of keypresses */
   int       kpstackpos;		/* location in stack (-1 = none) */
@@ -504,20 +508,28 @@ typedef struct _owl_obarray {
   owl_list strings;
 } owl_obarray;
 
-typedef struct _owl_dispatch {
-  int fd;                                 /* FD to watch for dispatch. */
+typedef struct _owl_io_dispatch {
+  int fd;                                     /* FD to watch for dispatch. */
+  int mode;
   int needs_gc;
-  void (*cfunc)(struct _owl_dispatch*);   /* C function to dispatch to. */
-  void (*destroy)(struct _owl_dispatch*); /* Destructor */
+  void (*callback)(const struct _owl_io_dispatch *, void *); /* C function to dispatch to. */
+  void (*destroy)(const struct _owl_io_dispatch *);  /* Destructor */
   void *data;
-} owl_dispatch;
+} owl_io_dispatch;
+
+typedef struct _owl_ps_action {
+  int needs_gc;
+  int (*callback)(struct _owl_ps_action *, void *);
+  void (*destroy)(struct _owl_ps_action *);
+  void *data;
+} owl_ps_action;
 
 typedef struct _owl_popexec {
   int refcount;
   owl_viewwin *vwin;
   int winactive;
-  int pid;			/* or 0 if it has terminated */
-  owl_dispatch dispatch;
+  pid_t pid;			/* or 0 if it has terminated */
+  const owl_io_dispatch *dispatch;
 } owl_popexec;
 
 typedef struct _owl_global {
@@ -526,7 +538,8 @@ typedef struct _owl_global {
   owl_history cmdhist;		/* command history */
   owl_history msghist;		/* outgoing message history */
   owl_keyhandler kh;
-  owl_list filterlist;
+  owl_dict filters;
+  GList *filterlist;
   owl_list puntlist;
   owl_vardict vars;
   owl_cmddict cmds;
@@ -534,6 +547,7 @@ typedef struct _owl_global {
   owl_errqueue errqueue;
   int lines, cols;
   int curmsg, topmsg;
+  int markedmsgid;              /* for finding the marked message when it has moved. */
   int curmsg_vert_offset;
   owl_view current_view;
   owl_messagelist msglist;
@@ -553,7 +567,7 @@ typedef struct _owl_global {
   int haveconfig;
   int config_format;
   void *buffercbdata;
-  owl_editwin tw;
+  owl_editwin *tw;
   owl_viewwin vw;
   void *perl;
   int debug;
@@ -565,10 +579,9 @@ typedef struct _owl_global {
   int hascolors;
   int colorpairs;
   owl_colorpair_mgr cpmgr;
-  int searchactive;
-  int newmsgproc_pid;
+  pid_t newmsgproc_pid;
   int malloced, freed;
-  char *searchstring;
+  owl_regex search_re;
   aim_session_t aimsess;
   aim_conn_t bosconn;
   owl_timer aim_noop_timer;
@@ -590,10 +603,12 @@ typedef struct _owl_global {
   owl_timer zephyr_buddycheck_timer;
   struct termios startup_tio;
   owl_obarray obarray;
-  owl_list dispatchlist;
+  owl_list io_dispatch_list;
+  owl_list psa_list;
   GList *timerlist;
   owl_timer *aim_nop_timer;
   int load_initial_subs;
+  int interrupted;
 } owl_global;
 
 /* globals */
