@@ -17,6 +17,7 @@ use BarnOwl;
 use BarnOwl::Hooks;
 use BarnOwl::Message::IRC;
 use BarnOwl::Module::IRC::Connection qw(is_private);
+use BarnOwl::Module::IRC::Completion;
 
 use Net::IRC;
 use Getopt::Long;
@@ -28,6 +29,7 @@ our $irc;
 # Hash alias -> BarnOwl::Module::IRC::Connection object
 our %ircnets;
 our %channels;
+our %reconnect;
 
 sub startup {
     BarnOwl::new_variable_string('irc:nick', {
@@ -49,17 +51,25 @@ sub startup {
         'originally intended to display your real name, which people '  .
         'often use for pithy quotes and URLs.'
        });
-    
+
     BarnOwl::new_variable_bool('irc:spew', {
         default     => 0,
         summary     => 'Show unhandled IRC events',
         description => 'If set, display all unrecognized IRC events as ' .
-        'admin messages. Intended for debugging and development use only '
+        'admin messages. Intended for debugging and development use only.'
        });
-    
+
+    BarnOwl::new_variable_string('irc:skip', {
+        default     => 'welcome yourhost created ' .
+        'luserclient luserme luserop luserchannels',
+        summary     => 'Skip messages of these types',
+        description => 'If set, each (space-separated) message type ' .
+        'provided will be hidden and ignored if received.'
+       });
+
     register_commands();
     register_handlers();
-    BarnOwl::filter('irc type ^IRC$');
+    BarnOwl::filter(qw{irc type ^IRC$ or ( type ^admin$ and adminheader ^IRC$ )});
 }
 
 sub shutdown {
@@ -77,6 +87,24 @@ Use ':irc-connect @b[server]' to connect to an IRC server, and
 END_QUICKSTART
 }
 
+sub buddylist {
+    my $list = "";
+
+    for my $net (sort keys %ircnets) {
+        my $conn = $ircnets{$net};
+        my ($nick, $server) = ($conn->nick, $conn->server);
+        $list .= BarnOwl::Style::boldify("IRC channels for $net ($nick\@$server)");
+        $list .= "\n";
+
+        for my $chan (keys %channels) {
+            next unless grep $_ eq $conn, @{$channels{$chan}};
+            $list .= "  $chan\n";
+        }
+    }
+
+    return $list;
+}
+
 #sub mainloop_hook {
 #    return unless defined $irc;
 #    eval {
@@ -101,6 +129,13 @@ sub register_handlers {
     }
 }
 
+sub skip_msg {
+    my $class = shift;
+    my $type = lc shift;
+    my $skip = lc BarnOwl::getvar('irc:skip');
+    return grep {$_ eq $type} split ' ', $skip;
+}
+
 use constant OPTIONAL_CHANNEL => 1;
 use constant REQUIRE_CHANNEL => 2;
 
@@ -111,10 +146,8 @@ sub register_commands {
             summary => 'Connect to an IRC server',
             usage =>
 'irc-connect [-a ALIAS ] [-s] [-p PASSWORD] [-n NICK] SERVER [port]',
-            description =>
-
-              <<END_DESCR
-Connect to an IRC server. Supported options are
+            description => <<END_DESCR
+Connect to an IRC server. Supported options are:
 
  -a <alias>          Define an alias for this server
  -s                  Use SSL
@@ -133,7 +166,7 @@ END_DESCR
     );
 
     BarnOwl::new_command(
-        'irc-disconnect' => mk_irc_command( \&cmd_disconnect ),
+        'irc-disconnect' => \&cmd_disconnect,
         {
             summary => 'Disconnect from an IRC server',
             usage   => 'irc-disconnect [-a ALIAS]',
@@ -152,7 +185,7 @@ END_DESCR
             usage   => 'irc-msg [-a ALIAS] DESTINATION MESSAGE',
 
             description => <<END_DESCR
-Send an IRC message
+Send an IRC message.
 END_DESCR
         }
     );
@@ -164,7 +197,6 @@ END_DESCR
             usage   => 'irc-mode [-a ALIAS] TARGET [+-]MODE OPTIONS',
 
             description => <<END_DESCR
-
 Change the mode of an IRC user or channel.
 END_DESCR
         }
@@ -177,7 +209,6 @@ END_DESCR
             usage   => 'irc-join [-a ALIAS] #channel [KEY]',
 
             description => <<END_DESCR
-
 Join an IRC channel.
 END_DESCR
         }
@@ -190,7 +221,6 @@ END_DESCR
             usage   => 'irc-part [-a ALIAS] #channel',
 
             description => <<END_DESCR
-
 Part from an IRC channel.
 END_DESCR
         }
@@ -203,7 +233,6 @@ END_DESCR
             usage   => 'irc-nick [-a ALIAS] NEW-NICK',
 
             description => <<END_DESCR
-
 Set your IRC nickname on an existing connect. To change it prior to
 connecting, adjust the `irc:nick' variable.
 END_DESCR
@@ -217,7 +246,6 @@ END_DESCR
             usage   => 'irc-names [-a ALIAS] #channel',
 
             description => <<END_DESCR
-
 `irc-names' displays the list of users in a given channel in a pop-up
 window.
 END_DESCR
@@ -231,7 +259,6 @@ END_DESCR
             usage   => 'irc-whois [-a ALIAS] NICK',
 
             description => <<END_DESCR
-
 Pops up information about a given IRC user.
 END_DESCR
         }
@@ -244,7 +271,6 @@ END_DESCR
             usage   => 'irc-motd [-a ALIAS]',
 
             description => <<END_DESCR
-
 Displays an IRC server's message of the day.
 END_DESCR
         }
@@ -257,7 +283,6 @@ END_DESCR
             usage   => 'irc-list',
 
             description => <<END_DESCR
-
 Show all the currently active IRC connections with their aliases and
 server names.
 END_DESCR
@@ -274,7 +299,6 @@ END_DESCR
             usage   => 'irc-topic [-a ALIAS] #channel [TOPIC]',
 
             description => <<END_DESCR
-
 Without extra arguments, fetches and displays a given channel's topic.
 
 With extra arguments, changes the target channel's topic string. This
@@ -290,7 +314,6 @@ END_DESCR
             usage   => 'irc-quote [-a ALIAS] TEXT',
 
             description => <<END_DESCR
-
 Send a raw command line to an IRC server.
 
 This can be used to perform some operation not yet supported by
@@ -304,6 +327,7 @@ END_DESCR
 $BarnOwl::Hooks::startup->add('BarnOwl::Module::IRC::startup');
 $BarnOwl::Hooks::shutdown->add('BarnOwl::Module::IRC::shutdown');
 $BarnOwl::Hooks::getQuickstart->add('BarnOwl::Module::IRC::quickstart');
+$BarnOwl::Hooks::getBuddyList->add("BarnOwl::Module::IRC::buddylist");
 
 ################################################################################
 ######################## Owl command handlers ##################################
@@ -357,11 +381,7 @@ sub cmd_connect {
        );
 
     if ($conn->conn->connected) {
-        BarnOwl::admin_message("IRC", "Connected to $alias as $nick");
-        $ircnets{$alias} = $conn;
-        my $fd = $conn->getSocket()->fileno();
-        BarnOwl::add_dispatch($fd, \&OwlProcess);
-        $conn->{FD} = $fd;
+        $conn->connected("Connected to $alias as $nick");
     } else {
         die("IRC::Connection->connect failed: $!");
     }
@@ -370,10 +390,27 @@ sub cmd_connect {
 }
 
 sub cmd_disconnect {
-    my $cmd = shift;
-    my $conn = shift;
-    $conn->conn->disconnect;
-    delete $ircnets{$conn->alias};
+    # Such a hack
+    local *get_connection_by_alias = sub {
+        my $key = shift;
+        return $ircnets{$key} if exists $ircnets{$key};
+        return $reconnect{$key}{conn} if exists $reconnect{$key};
+        die("No such ircnet: $key\n");
+    };
+
+    mk_irc_command(
+        sub {
+            my $cmd = shift;
+            my $conn = shift;
+            if ($conn->conn->connected) {
+                $conn->conn->disconnect;
+            } elsif ($reconnect{$conn->alias}) {
+                BarnOwl::admin_message('IRC',
+                                       "[" . $conn->alias . "] Reconnect cancelled");
+                delete $reconnect{$conn->alias};
+            }
+        }
+    )->(@_);
 }
 
 sub cmd_msg {
@@ -384,8 +421,9 @@ sub cmd_msg {
     if(@_) {
         process_msg($conn, $to, join(" ", @_));
     } else {
-        BarnOwl::start_edit_win("/msg -a " . $conn->alias . " $to", sub {process_msg($conn, $to, @_)});
+        BarnOwl::start_edit_win(BarnOwl::quote('/msg', '-a', $conn->alias, $to), sub {process_msg($conn, $to, @_)});
     }
+    return;
 }
 
 sub process_msg {
@@ -395,10 +433,10 @@ sub process_msg {
     # Strip whitespace. In the future -- send one message/line?
     $body =~ tr/\n\r/  /;
     if ($body =~ /^\/me (.*)/) {
-        $conn->conn->me($to, $1);
+        $conn->conn->me($to, Encode::encode('utf-8', $1));
         $body = '* '.$conn->nick.' '.$1;
     } else {
-        $conn->conn->privmsg($to, $body);
+        $conn->conn->privmsg($to, Encode::encode('utf-8', $body));
     }
     my $msg = BarnOwl::Message->new(
         type        => 'IRC',
@@ -410,10 +448,11 @@ sub process_msg {
         sender      => $conn->nick,
         is_private($to) ?
           (isprivate  => 'true') : (channel => $to),
-        replycmd    => "irc-msg -a " . $conn->alias . " $to",
-        replysendercmd => "irc-msg -a " . $conn->alias . " $to"
+        replycmd    => BarnOwl::quote('irc-msg',  '-a', $conn->alias, $to),
+        replysendercmd => BarnOwl::quote('irc-msg', '-a', $conn->alias, $to),
        );
     BarnOwl::queue_message($msg);
+    return;
 }
 
 sub cmd_mode {
@@ -422,6 +461,7 @@ sub cmd_mode {
     my $target = shift;
     $target ||= shift;
     $conn->conn->mode($target, @_);
+    return;
 }
 
 sub cmd_join {
@@ -431,6 +471,7 @@ sub cmd_join {
     $channels{$chan} ||= [];
     push @{$channels{$chan}}, $conn;
     $conn->conn->join($chan, @_);
+    return;
 }
 
 sub cmd_part {
@@ -439,6 +480,7 @@ sub cmd_part {
     my $chan = shift;
     $channels{$chan} = [grep {$_ ne $conn} @{$channels{$chan} || []}];
     $conn->conn->part($chan);
+    return;
 }
 
 sub cmd_nick {
@@ -446,6 +488,7 @@ sub cmd_nick {
     my $conn = shift;
     my $nick = shift or die("Usage: $cmd <new nick>\n");
     $conn->conn->nick($nick);
+    return;
 }
 
 sub cmd_names {
@@ -454,6 +497,7 @@ sub cmd_names {
     my $chan = shift;
     $conn->names_tmp([]);
     $conn->conn->names($chan);
+    return;
 }
 
 sub cmd_whois {
@@ -461,12 +505,14 @@ sub cmd_whois {
     my $conn = shift;
     my $who = shift || die("Usage: $cmd <user>\n");
     $conn->conn->whois($who);
+    return;
 }
 
 sub cmd_motd {
     my $cmd = shift;
     my $conn = shift;
     $conn->conn->motd;
+    return;
 }
 
 sub cmd_list {
@@ -476,6 +522,7 @@ sub cmd_list {
         $message .= '  ' . $alias . ' => ' . $conn->nick . '@' . $conn->server . "\n";
     }
     BarnOwl::popless_ztext($message);
+    return;
 }
 
 sub cmd_who {
@@ -484,6 +531,7 @@ sub cmd_who {
     my $who = shift || die("Usage: $cmd <user>\n");
     BarnOwl::error("WHO $cmd $conn $who");
     $conn->conn->who($who);
+    return;
 }
 
 sub cmd_stats {
@@ -491,6 +539,7 @@ sub cmd_stats {
     my $conn = shift;
     my $type = shift || die("Usage: $cmd <chiklmouy> [server] \n");
     $conn->conn->stats($type, @_);
+    return;
 }
 
 sub cmd_topic {
@@ -498,12 +547,14 @@ sub cmd_topic {
     my $conn = shift;
     my $chan = shift;
     $conn->conn->topic($chan, @_ ? join(" ", @_) : undef);
+    return;
 }
 
 sub cmd_quote {
     my $cmd = shift;
     my $conn = shift;
     $conn->conn->sl(join(" ", @_));
+    return;
 }
 
 ################################################################################
@@ -528,21 +579,20 @@ sub mk_irc_command {
         if(defined($alias)) {
             $conn = get_connection_by_alias($alias);
         }
-        if(!$conn && $use_channel) {
+        if($use_channel) {
             $channel = $ARGV[0];
             if(defined($channel) && $channel =~ /^#/) {
                 if($channels{$channel} && @{$channels{$channel}} == 1) {
                     shift @ARGV;
-                    $conn = $channels{$channel}[0];
-                }  
-            } else {
-                if($m && $m->type eq 'IRC' && !$m->is_private) {
-                    $channel = $m->channel;
-                } else {
-                    undef $channel;
+                    $conn = $channels{$channel}[0] unless $conn;
                 }
+            } elsif ($m && $m->type eq 'IRC' && !$m->is_private) {
+                $channel = $m->channel;
+            } else {
+                undef $channel;
             }
         }
+
         if(!$channel && $use_channel == REQUIRE_CHANNEL) {
             die("Usage: $cmd <channel>\n");
         }
